'use strict';

require('babel-register')({
  "presets": [require.resolve('babel-preset-es2015'), require.resolve('babel-preset-stage-0')]
});

var fs = require('fs');
var path = require('path');

var Logger = require('./logger');
var config = require('../config.tmp.json');

var schemaPath = config.schema.entry;
var graphqlPath = path.join(config.rootPath, '/node_modules/graphql');
var graphqlUtilitiesPath = path.join(graphqlPath, '/utilities');

function cleanCache() {
  for (var key in require.cache) {
    if (key.indexOf(config.schema.watch) > -1) {
      delete require.cache[key];
    }
  }
}

module.exports = function () {
  Logger.debug('Updating schema at: ' + schemaPath);

  cleanCache();

  var Schema = require(schemaPath).default;

  var _require = require(graphqlPath),
      graphql = _require.graphql;

  var _require2 = require(graphqlUtilitiesPath),
      introspectionQuery = _require2.introspectionQuery,
      printSchema = _require2.printSchema;

  return graphql(Schema, introspectionQuery).then(function (result) {
    if (result.errors) {
      var errors = JSON.stringify(result.errors, null, 2);
      console.error('ERROR introspecting schema: ', errors);
      return result;
    }

    Logger.debug('Writing schema .json at: ' + config.schema.json);

    fs.writeFileSync(config.schema.json, JSON.stringify(result, null, 2));

    Logger.debug('Writing schema .graphql at: ' + config.schema.json);

    fs.writeFileSync(config.schema.graphql, printSchema(Schema));

    Logger.debug('Schema updated.');

    return result;
  }).catch(function (err) {
    return console.log(err);
  });
};